"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.esbuildPluginAliasPath = void 0;
const normalize_options_1 = require("./normalize-options");
const typescript_1 = require("typescript");
const escape_namespace_1 = require("./escape-namespace");
const path_1 = __importDefault(require("path"));
// const debug = require('debug')('esbuild:alias-path');
const pluginName = 'esbuild:alias-path';
const esbuildPluginAliasPath = (options = {}) => {
    const { alias, skip, tsconfigPath, compilerOptions } = normalize_options_1.normalizeOption(options);
    if (skip) {
        return {
            name: pluginName,
            setup(build) {
                void 0;
            },
        };
    }
    const escapedNamespace = escape_namespace_1.escapeNamespace(Object.keys(alias));
    return {
        name: pluginName,
        setup(build) {
            // handle alias replacement
            build.onResolve({ filter: escapedNamespace }, ({ path: fromPath }) => {
                const replacedPath = alias[fromPath];
                if (!replacedPath) {
                    return null;
                }
                return {
                    path: path_1.default.isAbsolute(replacedPath)
                        ? replacedPath
                        : path_1.default.join(process.cwd(), replacedPath),
                };
            });
            // handle tsconfig paths mapping
            build.onResolve({ filter: /.*/ }, ({ path: filePath, importer }) => __awaiter(this, void 0, void 0, function* () {
                if (!compilerOptions.paths) {
                    return null;
                }
                const hasMatchingPath = Object.keys(compilerOptions.paths).some((path) => new RegExp(path.replace('*', '\\w*')).test(filePath));
                if (!hasMatchingPath) {
                    return null;
                }
                const { resolvedModule } = typescript_1.nodeModuleNameResolver(filePath, importer, compilerOptions, typescript_1.sys);
                const { resolvedFileName } = resolvedModule;
                if (!resolvedFileName || resolvedFileName.endsWith('.d.ts')) {
                    return null;
                }
                const resolved = typescript_1.sys.resolvePath(resolvedFileName);
                return {
                    path: resolved,
                };
            }));
        },
    };
};
exports.esbuildPluginAliasPath = esbuildPluginAliasPath;
